From 6a39cee30848489b13ddef026a13706353145262 Mon Sep 17 00:00:00 2001
From: Philipp Stephani <phst@google.com>
Date: Sun, 22 Nov 2015 20:52:55 +0100
Subject: [PATCH 2/2] Add functions to access the Emacs allocator

* emacs-module.c (module_allocate_raw_memory)
(module_free_raw_memory): new functions
---
 src/emacs-module.c | 21 +++++++++++++++++++++
 src/emacs-module.h | 11 +++++++++++
 2 files changed, 32 insertions(+)

diff --git a/src/emacs-module.c b/src/emacs-module.c
index 8705280..1f4e3f5 100644
--- a/src/emacs-module.c
+++ b/src/emacs-module.c
@@ -287,6 +287,25 @@ module_make_global_ref (emacs_env *env, emacs_value ref)
   return allocate_emacs_value (&global_storage, new_obj);
 }
 
+static void *
+module_allocate_raw_memory (emacs_env *env, ptrdiff_t size)
+{
+  check_main_thread ();
+  eassert (module_non_local_exit_check (env) == emacs_funcall_exit_return);
+  MODULE_HANDLE_SIGNALS;
+  return xmalloc (size);
+}
+
+static void
+module_free_raw_memory (emacs_env *env, void *memory)
+{
+  check_main_thread ();
+  eassert (module_non_local_exit_check (env) == emacs_funcall_exit_return);
+  MODULE_HANDLE_SIGNALS_VOID;
+  eassert (memory != NULL);
+  return xfree (memory);
+}
+
 static void
 module_free_global_ref (emacs_env *env, emacs_value ref)
 {
@@ -953,6 +972,8 @@ initialize_environment (struct env_storage *env)
   env->pub.private_members = &env->priv;
   env->pub.make_global_ref = module_make_global_ref;
   env->pub.free_global_ref = module_free_global_ref;
+  env->pub.allocate_raw_memory = module_allocate_raw_memory;
+  env->pub.free_raw_memory = module_free_raw_memory;
   env->pub.non_local_exit_check = module_non_local_exit_check;
   env->pub.non_local_exit_clear = module_non_local_exit_clear;
   env->pub.non_local_exit_get = module_non_local_exit_get;
diff --git a/src/emacs-module.h b/src/emacs-module.h
index 06fc4c0..ecead23 100644
--- a/src/emacs-module.h
+++ b/src/emacs-module.h
@@ -95,6 +95,17 @@ struct emacs_env_25
   void (*free_global_ref) (emacs_env *env,
 			   emacs_value global_reference);
 
+  /* Uses the Emacs memory allocator to allocate raw memory.  Returns
+     NULL if allocation fails.  Using this function over malloc(3) has
+     the advantage that input is correctly blocked and the memory
+     profiler is notified about the allocation.  Memory allocated with
+     this function must be freed by free_raw_memory.  */
+  void *(*allocate_raw_memory) (emacs_env *env, ptrdiff_t size);
+
+  /* Frees a block of memory allocated by allocate_raw_memory.  The
+     pointer to the memory block must not be NULL.  */
+  void (*free_raw_memory) (emacs_env *env, void *memory);
+
   /* Non-local exit handling.  */
 
   enum emacs_funcall_exit (*non_local_exit_check) (emacs_env *env);
-- 
2.6.3

